'use strict';

app.controller('StatusCtrl', ['$scope', 'FormService', 'LibraryService', 'TranslateService', 'RestService', 'FormDataService',
    'formData', 'users', 'profiles', 'networkInterfaces', 'playbackStatus', '$templateCache',
	function ($scope, FormService, LibraryService, TranslateService, RestService, FormDataService, formData,
              users, profiles, networkInterfaces, playbackStatus, $templateCache) {
	
	$scope.formData = buildFormData(formData);
    $scope.users = fixUsers(users);
    $scope.profiles = profiles;
    $scope.networkInterfaces = buildNetworkInterfaces(networkInterfaces);
    $scope.playbackStatus = playbackStatus['clients'];
    $scope.serverStatus = $scope.formData['serverStatus'];

    $scope.editRendererOpen = false;
    $scope.editingRendererIndex = null;
    $scope.editedRenderer = null;
    $scope.focusedPlaybackStatus = null;

    $scope.playbackPopoverTemplate = "playbackStatus.html";

    $templateCache.put('playbackStatus.html',
        '<div class="playbackStatusContainer">' +
        '<div class="thumbnailContainer hidden-xs">' +
        '<img src="assets/images/thumbnail-shape.png" class="thumbnailImg" style="background: url({{focusedPlaybackStatus.itemThumbnailUrl}}) 50% 50% no-repeat;">' +
        '</div>' +
        '<div class="description">' +
        '<div class="title">{{focusedPlaybackStatus.itemTitle}}</div>' +
        '<div class="progress-xs progress margin-top-15">' +
        '<div class="progress-bar progress-bar-success" role="progressbar" aria-valuenow="{{focusedPlaybackStatus.playbackProgress}}" aria-valuemin="0" aria-valuemax="100" style="width: {{focusedPlaybackStatus.playbackProgress}}%"></div>' +
        '</div>' +
        '</div>' +
        '</div>');

    var originalRenderers = angular.copy($scope.formData.renderers);

    function resetEditForm() { 
        $scope.editingRendererIndex = null;
        $scope.$broadcast("resettableFormReset"); // mark the inner form as pristine
    }

    function closeEditForm() {
        $scope.editRendererOpen = false;
    }

    function fixUsers(users) {
        var updatedUsers = [];
        updatedUsers[0] = {name: -1, value: TranslateService.getMessage('unassigned')};
        for(var i = 0; i < users.length; i++) {
            updatedUsers[i + 1] = {name: parseInt(users[i].name), value: users[i].value};    
        }
        return updatedUsers;
    }

    function buildFormData(formData) {
        if(! ('boundNICName' in formData)) {
            formData['boundNICName'] = 0;
        }

        setUsersOnRenderers(formData);
        
        return formData;
    }

    function setUsersOnRenderers(formData) {
        var renderers = formData['renderers'];
        for(var i = 0; i < renderers.length; i++ ) {
            var renderer = renderers[i];
            if(! ('userId' in renderer)) {
                renderer['userId'] = -1;
            }
        }
    }
    
    function removeUsersFromRenderers(formData) {
        var renderers = formData['renderers'];
        for(var i = 0; i < renderers.length; i++ ) {
            var renderer = renderers[i];
            if(renderer['userId'] == -1) {
                delete renderer['userId'];
            }
        }
    }
    
    function buildNetworkInterfaces(networkInterfaces) {
        var nics = [];
        nics.push({name: 0, value: TranslateService.getMessage('tab_status_bound_ip_address_automatically_detected')});
        for(var i = 0; i < networkInterfaces.length; i++) {
            nics.push( networkInterfaces[i] );
        }
        return nics;
    }

    function presaveFormData(formData) {
        var formCopy = angular.copy(formData);
        if(formCopy.boundNICName == 0) {
            delete formCopy['boundNICName'];
        }
        if(formCopy.friendlyName == '') {
            delete formCopy['friendlyName'];
        }

        removeUsersFromRenderers(formCopy);

        return formCopy;
    }

    $scope.$on("formStateReset", function (event, modified) {
        $scope.formData['renderers'] = angular.copy(originalRenderers);     
        LibraryService.markAsPristine($scope, null);
        resetEditForm();          
    });

    $scope.submit = function (form) {
        FormService.submitForm('status', presaveFormData($scope.formData), form, $scope);

        originalRenderers = angular.copy($scope.formData.renderers);

        LibraryService.markAsPristine($scope, form);
    };

    $scope.reloadRenderers = function() {
        FormDataService.loadFormData('status').then(function(json) {
            $scope.formData['renderers'] = json.renderers;
            setUsersOnRenderers($scope.formData);
            originalRenderers = angular.copy($scope.formData.renderers);
        });

        FormDataService.loadFormData('playback-status', true).then(function(json) {
            $scope.playbackStatus = json['clients'];
        });
    };

    $scope.profileName = function(profileId) {
        for(var i = 0; i < $scope.profiles.length; i++) {
            var profile = $scope.profiles[i];
            if(profile.name == profileId) {
                return profile.value;
            }
        }
        return profileId;
    };

    $scope.userName = function(userId) {
        for(var i = 0; i < $scope.users.length; i++) {
            var user = $scope.users[i];
            if(user.name == userId) {
                return user.value;
            }
        }
        return userId;
    };
    
    $scope.rendererName = function(renderer) {
        if(renderer.friendlyName) {
            return renderer.friendlyName;
        } else {
            return renderer.name;
        }
    };

    $scope.enableOrDisableRenderer = function(index, enabled, form) {
        $scope.formData.renderers[index].enabled = enabled;
        LibraryService.markAsModified($scope, form);
    };


    $scope.removeRenderer = function(index, form) {
        var renderers = $scope.formData.renderers;
        renderers.splice(index, 1);
        LibraryService.markAsModified($scope, form);
    };

    $scope.openEditRendererForm = function(index) {
        $scope.editedRenderer = angular.copy($scope.formData.renderers[index]);
        $scope.editRendererOpen = true;
        $scope.editingRendererIndex = index;
    };

    $scope.updateRenderer = function(form, index) {
        var valid = FormService.validateForm(form);
        if(valid) {
            var newSource = angular.copy($scope.editedRenderer);
            $scope.formData.renderers[index] = newSource;
            LibraryService.markAsModified($scope, form.$$parentForm);
            closeEditForm();
            resetEditForm();
        }     
    };

    $scope.cancelUpdateRenderer = function() {
        closeEditForm();
        resetEditForm();
    };

    $scope.startStopServer = function() {
        $scope.serverStartStopInProcess = true;
        if($scope.serverStatus == 'STARTED') {
            RestService.invokeAction('stopServer', null, true, function(response) {
                $scope.serverStatus = 'STOPPED';
                $scope.serverStartStopInProcess = false;
            }, function(response) {
                $scope.serverStartStopInProcess = false;
            });
        } else {
            RestService.invokeAction('startServer', null, true, function(response) {
                $scope.serverStatus = 'STARTED';
                $scope.serverStartStopInProcess = false;
            }, function(response) {
                $scope.serverStartStopInProcess = false;
            });
        }
    };

    $scope.playbackStatusForRenderer = function(renderer) {
        var rendererPlaybackStatus = $scope.playbackStatus.find(function(ps) { 
            return ps['rendererUuid'] && ps['rendererUuid'] == renderer.uuid;
        });
        return rendererPlaybackStatus;
    };

    $scope.setFocusedPlaybackStatus = function(playbackStatus) {
        $scope.focusedPlaybackStatus = playbackStatus;
    }

}]);
