'use strict';

app.controller('UsersCtrl', ['$scope', 'FormService', 'FormDataService', 'LibraryService', 'TranslateService', 'formData', 'ratings',
	function ($scope, FormService, FormDataService, LibraryService, TranslateService, formData, ratings) {
	
	$scope.formData = formData;

	var cleanAddForm = {
        name: '',
        password: ''
    };

    $scope.ratings = ratings;
    $scope.addUserOpen = false;
	$scope.newUser = angular.copy(cleanAddForm);
    $scope.editingUserIndex = null;
    $scope.editingUser = false;
	var originalUsers = angular.copy($scope.formData.users);

    function resetAddForm() {
        $scope.editingUserIndex = null;
        $scope.newUser = angular.copy(cleanAddForm);
        $scope.$broadcast("resettableFormReset"); // mark the inner form as pristine
    }

    function closeAddUserForm() {
        $scope.addUserOpen = false;
    }

    $scope.$on("formStateReset", function (event, modified) {
        $scope.formData['users'] = angular.copy(originalUsers);
        LibraryService.markAsPristine($scope, null);
        resetAddForm();          
    });

    $scope.submit = function (form) {
        FormService.submitForm('users', $scope.formData, form, $scope, function(response) {
            FormDataService.loadFormData('users').then(function(json) {
                $scope.formData['users'] = json.users;
                originalUsers = angular.copy($scope.formData.users);

                LibraryService.markAsPristine($scope, form);
            });    
        });
    };

    $scope.openAddUserForm = function() {
        $scope.newUser = prepareNewUserModel(angular.copy(cleanAddForm));
        $scope.addUserOpen = true;
        $scope.editingUser = false;
    };

    $scope.openEditUserForm = function(index) {
        $scope.newUser = prepareNewUserModel(angular.copy($scope.formData.users[index]));
        $scope.addUserOpen = true;
        $scope.editingUserIndex = index;
        $scope.editingUser = true;
    };

    $scope.cancelAddUser = function() {
        closeAddUserForm();
        resetAddForm();
    };

    $scope.addUser = function(form, index) {
        var valid = FormService.validateForm(form);
        if(valid) {
            var newUser = normalizeNewUserModel(angular.copy($scope.newUser));
            if(index != null) {
                // edit
                $scope.formData.users[index] = newUser;
            } else {
                // new
                $scope.formData.users.push(newUser);
            }

            LibraryService.markAsModified($scope, form.$$parentForm);
            closeAddUserForm();
            resetAddForm();
        }     
    };

    $scope.removeUser = function(index, form) {
        var users = $scope.formData.users;
        users.splice(index, 1);
        LibraryService.markAsModified($scope, form);
    };
    
    $scope.ratingForUser = function(user) {
        if(user.maxAllowedMPAARating) {
            return $scope.ratings.find(function (rating) {
                return user.maxAllowedMPAARating == rating.name;
            }).value;
        } else {
            return TranslateService.getMessage('unassigned');
        }
    };
    
    $scope.userChangedName = function(user) {
        var existingUserId = user['id'];
        if(existingUserId) {
            var originalUser = originalUsers.find(function(u) {
                return u['id'] == existingUserId;
            });
            if(originalUser && originalUser['name'] != user['name']) {
                return true;
            } else {
                return false;
            }
        } else {
            // newly added user
            return false;
        }
    };

    function prepareNewUserModel(model) {
        if('maxAllowedMPAARating' in model) {
            model['maxAllowedMPAARatingEnabled'] = true;
        } else {
            model['maxAllowedMPAARatingEnabled'] = false;
            model['maxAllowedMPAARating'] = 'NC17';
        }
    	return model;
    }

    function normalizeNewUserModel(model) {
        if(model.maxAllowedMPAARatingEnabled == false) {
            delete model['maxAllowedMPAARating'];
        }
        delete model['maxAllowedMPAARatingEnabled'];
        return model;
    }

}]);